import { reservedWords } from "./reservedWords.js";
/**
 * This is the secret to our safety; since this is a symbol it cannot be faked
 * in a JSON payload and it cannot be constructed with a new Symbol (even with
 * the same argument), so external data cannot make itself trusted.
 */
declare const $$type: unique symbol;
/**
 * Represents raw TE, the text will be output verbatim into the compiled code.
 */
export interface TERawNode {
    readonly [$$type]: "RAW";
    /** text */
    readonly t: string;
}
/**
 * Represents an TE value that will be replaced with a closure variable in the
 * compiled TE statement.
 */
export interface TERefNode {
    readonly [$$type]: "REF";
    /** value */
    readonly v: any;
}
export interface TETemporaryVariableNode {
    readonly [$$type]: "VARIABLE";
    /** symbol */
    readonly s: symbol;
}
/**
 * Represents that the TE inside this should be indented when pretty printed.
 */
export interface TEIndentNode {
    readonly [$$type]: "INDENT";
    /** content */
    readonly c: TEQuery;
}
/** @internal */
export declare type TENode = TERawNode | TERefNode | TETemporaryVariableNode | TEIndentNode;
/** @internal */
export interface TEQuery {
    readonly [$$type]: "QUERY";
    /** nodes */
    readonly n: ReadonlyArray<TENode>;
}
/**
 * Representation of TE, identifiers, values, etc; to generate a query that
 * can be issued to the database it needs to be fed to `te.compile`.
 */
export declare type TE = TENode | TEQuery;
declare function isTE(node: unknown): node is TE;
/**
 * Accepts an te`...` expression and compiles it out to TE text with
 * placeholders, and the values to substitute for these values.
 */
declare function compile(fragment: TE): {
    string: string;
    refs: {
        [key: string]: any;
    };
};
/**
 * Creates a TE item for a raw code string. Just plain ol‘ raw code. This
 * method is dangerous though because it involves no escaping, so proceed with
 * caution! It's very very rarely warranted - there is likely a safer way of
 * achieving your goal. DO NOT USE THIS WITH UNTRUSTED INPUT!
 */
declare function raw(text: string): TE;
/**
 * Creates a TE item for a value that will be included in our final query.
 * This value will be added in a way which avoids TE injection.
 */
declare function ref(val: any): TE;
declare const undefinedNode: TERawNode;
export declare function stringifyString(value: string): string;
export declare const toJSON: (value: any) => string;
/**
 * If the value is simple will inline it into the query, otherwise will defer
 * to `te.ref`.
 */
declare function lit(val: any): TE;
/**
 * If you're building a string and you want to inject untrusted content into it
 * without opening yourself to code injection attacks, this is the method for
 * you. Example:
 *
 * ```js
 * const code = te`const str = "abc${te.substring(untrusted, '"')}123";`
 * ```
 */
declare function substring(text: string, stringType: "'" | '"' | "`"): TE;
/**
 * Escapes `content` so that it can be safely embedded in a multiline comment.
 */
declare function subcomment(content: string | number | null | undefined): TERawNode;
/**
 * Is safe to set as the key of a POJO (without a null prototype).
 */
export declare const isSafeObjectPropertyName: (key: string | symbol | number) => boolean;
/**
 * Can represent as an identifier rather than a string key
 *
 * @remarks
 * Doesn't allow it to start with two underscores.
 */
export declare const canRepresentAsIdentifier: (key: string | symbol | number) => key is string | number;
declare function identifier(name: string): TERawNode;
/**
 * IMPORTANT: It's strongly recommended that instead of defining an object via
 * `const obj = { ${te.dangerousKey(untrustedKey)}: value }` you instead use
 * `const obj = Object.create(null);` and then set the properties on the resulting
 * object via `${obj}[${te.lit(untrustedKey)}] = value;` - this prevents attacks such as
 * **prototype polution** since properties like `__proto__` are not special on
 * null-prototype objects, whereas they can cause havok in regular `{}` objects.
 */
declare function dangerousKey(key: string | symbol | number): TE;
/**
 * Accesses the key of an object either via `.` or `[]` as appropriate;
 * `obj${te.get(key)}` would become `obj.foo` or `obj["1foo"]` as
 * appropriate.
 */
declare function get(key: string | symbol | number): TE;
/**
 * Accesses the key of an object via optional-chaining:
 * `obj${te.optionalGet(key)}` would become `obj?.foo` or `obj?.["1foo"]` as
 * appropriate.
 */
declare function optionalGet(key: string | symbol | number): TE;
/**
 * Sets the key of an object either via `.` or `[]` as appropriate;
 * `obj${te.set(key)}` would become `obj.foo` or `obj["1foo"]` as
 * appropriate.
 *
 * If the object you're setting properties on has a `null` prototype
 * (`Object.create(null)`) then you can set `hasNullPrototype` to true and all
 * keys are allowed. If this is not the case, then an error will be thrown on
 * certain potentially dangerous keys such as `__proto__` or `constructor`.
 */
declare function set(key: string | symbol | number, hasNullPrototype?: boolean): TE;
/**
 * @experimental
 */
declare function tempVar(symbol?: symbol): TE;
declare function tmp(obj: TE, callback: (tmp: TE) => TE): TE;
declare function run<TResult>(fragment: TE): TResult;
declare function run<TResult>(strings: TemplateStringsArray, ...values: TE[]): TResult;
/**
 * Join some TE items together, optionally separated by a string. Useful when
 * dealing with lists of TE items, for example a dynamic list of columns or
 * variadic TE function arguments.
 */
declare function join(items: Array<TE>, separator?: string): TE;
/**
 * WARNING: all lines will be indented, without any parsing, so if there are
 * template literals that contain newlines, spaces will be added inside these
 * too.
 */
declare function indent(fragment: TE): TE;
declare function indent(strings: TemplateStringsArray, ...values: Array<TE>): TE;
declare function indentIf(condition: boolean, fragment: TE): TE;
declare const te: TamedEvil;
export default te;
export { compile, dangerousKey, run as eval, get, identifier, isTE, join, lit, lit as literal, optionalGet, ref, reservedWords, run, set, subcomment, substring, te, tempVar, tmp, undefinedNode as undefined, raw, };
export interface TamedEvil {
    (strings: TemplateStringsArray, ...values: Array<TE>): TE;
    te: TamedEvil;
    ref: typeof ref;
    reference: typeof ref;
    lit: typeof lit;
    literal: typeof lit;
    substring: typeof substring;
    subcomment: typeof subcomment;
    join: typeof join;
    identifier: typeof identifier;
    dangerousKey: typeof dangerousKey;
    get: typeof get;
    optionalGet: typeof optionalGet;
    set: typeof set;
    tmp: typeof tmp;
    tempVar: typeof tempVar;
    run: {
        <TResult>(fragment: TE): TResult;
        <TResult>(strings: TemplateStringsArray, ...values: TE[]): TResult;
    };
    eval: {
        <TResult>(fragment: TE): TResult;
        <TResult>(strings: TemplateStringsArray, ...values: TE[]): TResult;
    };
    compile: typeof compile;
    indent: typeof indent;
    indentIf: typeof indentIf;
    undefined: TE;
    blank: TE;
    isTE: typeof isTE;
    reservedWords: typeof reservedWords;
    raw: typeof raw;
}
//# sourceMappingURL=index.d.ts.map